"""Define constants for GUI."""

# Supported wafer map formats
SUPPORTED_FORMATS = [
    "SEMI G85-1101",
]

# GUI control
WINDOW_SIZE = (1100, 900)
VIEWER_SIZE = (800, 800)
ASPECT_RATIO = 1.35
WINDOW_SCALE = 0.65
VIEWER_SCALE = 0.6
DATA_SCALE = 0.4
GRID_SCALE = 0.3
LEGEND_SCALE = 0.7
BORDER_SIZE = 10
NULL_COLOR = "#666666"
PASS_COLOR = "#66CC00"
FAIL_COLOR = "#CC3300"

# Wafer data keys
WAFER_ID = "wafer_id"
LOT_ID = "lot_id"
WAFER_SIZE = "wafer_size"
CHIP_SIZE = "chip_size"
PRODUCT_ID = "product_id"
CREATE_DATE = "created"
ALL_KEYS = [WAFER_ID, LOT_ID, CHIP_SIZE, PRODUCT_ID, CREATE_DATE]

# Various constants
PASS = "Pass"
FAIL = "Fail"
NULL = "NULL"
DEFAULT_WAFER_SIZE = 300



"""Tests for wafermap module."""

import unittest
from waferview import wafermap
from waferview.gui.constants import (
    SUPPORTED_FORMATS,
    WAFER_ID,
    LOT_ID,
    WAFER_SIZE,
    CHIP_SIZE,
    PRODUCT_ID,
    CREATE_DATE,
    DEFAULT_WAFER_SIZE,
)


TEST_XML = "./xml/SEMI_G85/SEMI_G85_1101_MIN.xml"


class TestWaferMap(unittest.TestCase):
    """Test the WaferMap class."""

    def setUp(self):
        """Set up a WaferMap instance."""
        self.test_wmap = MockWaferMap()

    def tearDown(self):
        """Tear down a WaferMap instance."""
        self.test_wmap = None

    def test_check_format_ok(self):
        """Test the check_format method."""
        self.test_wmap._map_data = {"@FormatRevision": SUPPORTED_FORMATS[0]}
        self.test_wmap.check_format()
        self.assertEqual(self.test_wmap.format, SUPPORTED_FORMATS[0])
        self.assertTrue(self.test_wmap.is_valid)

    def test_check_format_ng(self):
        """Test the check_format method with bad versioning."""
        self.test_wmap._map_data = {"@FormatRevision": "foobar"}
        self.test_wmap.check_format()
        self.assertFalse(self.test_wmap.is_valid)

    def test_check_get_attributes_min(self):
        """Test the get_attributes method."""
        self.test_wmap._map_data = {"Device": {"@Rows": "13", "@Columns": "17"}}
        self.test_wmap.get_attributes()
        self.assertEqual(self.test_wmap.device_attr[WAFER_ID], None)
        self.assertEqual(self.test_wmap.device_attr[LOT_ID], None)
        self.assertEqual(self.test_wmap.device_attr[WAFER_SIZE], DEFAULT_WAFER_SIZE)
        self.assertEqual(self.test_wmap.device_attr[CHIP_SIZE], [0, 0])
        self.assertEqual(self.test_wmap.device_attr[PRODUCT_ID], None)
        self.assertEqual(self.test_wmap.device_attr[CREATE_DATE], None)
        self.assertEqual(self.test_wmap.device_attr["rows"], 13)
        self.assertEqual(self.test_wmap.device_attr["cols"], 17)

    def test_check_get_attributes_all(self):
        """Test the get_attributes method with everything extracted."""
        self.test_wmap._map_data = {
            "@WaferId": "foo",
            "Device": {
                "@LotId": "bar",
                "@WaferSize": "100",
                "@DeviceSizeX": "42",
                "@DeviceSizeY": "24",
                "@ProductId": "deadbeef",
                "@Rows": "13",
                "@Columns": "17",
                "@CreateDate": "19700101",
            },
        }
        self.test_wmap.get_attributes()
        self.assertEqual(self.test_wmap.device_attr[WAFER_ID], "foo")
        self.assertEqual(self.test_wmap.device_attr[LOT_ID], "bar")
        self.assertEqual(self.test_wmap.device_attr[WAFER_SIZE], 100)
        self.assertEqual(self.test_wmap.device_attr[CHIP_SIZE], [42, 24])
        self.assertEqual(self.test_wmap.device_attr[PRODUCT_ID], "deadbeef")
        self.assertEqual(self.test_wmap.device_attr[CREATE_DATE], "19700101")
        self.assertEqual(self.test_wmap.device_attr["rows"], 13)
        self.assertEqual(self.test_wmap.device_attr["cols"], 17)

    def test_get_codes_min(self):
        """Test the get_codes method with minimal input."""
        self.test_wmap._map_data = {
            "Device": {
                "@NullBin": "255",
                "Bin": [
                    {"@BinCode": "000", "@BinQuality": "Pass", "@BinCount": "100"},
                    {"@BinCode": "111", "@BinQuality": "Fail", "@BinCount": "10"},
                    {"@BinCode": "222", "@BinQuality": "Fail", "@BinCount": "1"},
                ],
            }
        }

        self.test_wmap.get_codes()

        self.assertTrue("000" in self.test_wmap.bin_codes)
        self.assertTrue("111" in self.test_wmap.bin_codes)
        self.assertTrue("222" in self.test_wmap.bin_codes)
        self.assertTrue("255" in self.test_wmap.bin_codes)

        self.assertTrue(self.test_wmap.bin_codes["000"]["status"])
        self.assertFalse(self.test_wmap.bin_codes["111"]["status"])
        self.assertFalse(self.test_wmap.bin_codes["222"]["status"])
        self.assertEqual(self.test_wmap.bin_codes["255"]["status"], None)

        self.assertEqual(self.test_wmap.bin_codes["000"]["desc"], "0")
        self.assertEqual(self.test_wmap.bin_codes["111"]["desc"], "1")
        self.assertEqual(self.test_wmap.bin_codes["222"]["desc"], "2")
        self.assertEqual(self.test_wmap.bin_codes["255"]["desc"], "NULL")

        self.assertEqual(self.test_wmap.bin_codes["000"]["count"], "100")
        self.assertEqual(self.test_wmap.bin_codes["111"]["count"], "10")
        self.assertEqual(self.test_wmap.bin_codes["222"]["count"], "1")
        self.assertEqual(self.test_wmap.bin_codes["255"]["count"], None)

    def test_get_codes_all(self):
        """Test the get_codes method with all input."""
        self.test_wmap._map_data = {
            "Device": {
                "@NullBin": "255",
                "Bin": [
                    {
                        "@BinCode": "000",
                        "@BinQuality": "Pass",
                        "@BinCount": "100",
                        "@BinDescription": "foo",
                    },
                    {
                        "@BinCode": "111",
                        "@BinQuality": "Fail",
                        "@BinCount": "10",
                        "@BinDescription": "bar",
                    },
                    {
                        "@BinCode": "222",
                        "@BinQuality": "Fail",
                        "@BinCount": "1",
                        "@BinDescription": "beef",
                    },
                    {
                        "@BinCode": "255",
                        "@BinQuality": "NULL",
                        "@BinCount": "0",
                        "@BinDescription": "Null bin",
                    },
                ],
            }
        }

        self.test_wmap.get_codes()

        self.assertTrue("000" in self.test_wmap.bin_codes)
        self.assertTrue("111" in self.test_wmap.bin_codes)
        self.assertTrue("222" in self.test_wmap.bin_codes)
        self.assertTrue("255" in self.test_wmap.bin_codes)

        self.assertTrue(self.test_wmap.bin_codes["000"]["status"])
        self.assertFalse(self.test_wmap.bin_codes["111"]["status"])
        self.assertFalse(self.test_wmap.bin_codes["222"]["status"])
        self.assertEqual(self.test_wmap.bin_codes["255"]["status"], None)

        self.assertEqual(self.test_wmap.bin_codes["000"]["desc"], "foo")
        self.assertEqual(self.test_wmap.bin_codes["111"]["desc"], "bar")
        self.assertEqual(self.test_wmap.bin_codes["222"]["desc"], "beef")
        self.assertEqual(self.test_wmap.bin_codes["255"]["desc"], "Null bin")

        self.assertEqual(self.test_wmap.bin_codes["000"]["count"], "100")
        self.assertEqual(self.test_wmap.bin_codes["111"]["count"], "10")
        self.assertEqual(self.test_wmap.bin_codes["222"]["count"], "1")
        self.assertEqual(self.test_wmap.bin_codes["255"]["count"], "0")

    def test_gen_map_dec_bins(self):
        """Test the gen_map method."""
        self.test_wmap.device_attr = {}
        self.test_wmap.device_attr[CHIP_SIZE] = [1, 1]
        self.test_wmap.device_attr["rows"] = 2
        self.test_wmap.device_attr["cols"] = 2
        self.test_wmap.bin_codes = {
            "000": {"status": True, "desc": None, "count": "1"},
            "111": {"status": False, "desc": None, "count": "2"},
            "222": {"status": None, "desc": "NULL", "count": "3"},
        }
        self.test_wmap._map_data = {
            "Device": {
                "@BinType": "Decimal",
                "Data": {
                    "Row": ["000111", "111222"],
                },
            }
        }

        self.test_wmap.gen_map()
        self.assertTrue([(0, 0.5), (0.5, 0.5), True, None] in self.test_wmap.pixels)
        self.assertTrue([(0.5, 0.5), (0.5, 0.5), False, None] in self.test_wmap.pixels)
        self.assertTrue([(0, 0), (0.5, 0.5), False, None] in self.test_wmap.pixels)
        self.assertTrue([(0.5, 0), (0.5, 0.5), None, "NULL"] in self.test_wmap.pixels)

    def test_gen_map_dec_bins_min(self):
        """Test the gen_map method with no chip size."""
        self.test_wmap.device_attr = {}
        self.test_wmap.device_attr[CHIP_SIZE] = [0, 0]
        self.test_wmap.device_attr[WAFER_SIZE] = 0.002
        self.test_wmap.device_attr["rows"] = 2
        self.test_wmap.device_attr["cols"] = 2
        self.test_wmap.bin_codes = {
            "000": {"status": True, "desc": None, "count": "1"},
            "111": {"status": False, "desc": None, "count": "2"},
            "222": {"status": None, "desc": "NULL", "count": "3"},
        }
        self.test_wmap._map_data = {
            "Device": {
                "@BinType": "Decimal",
                "Data": {
                    "Row": ["000111", "111222"],
                },
            }
        }

        self.test_wmap.gen_map()
        self.assertTrue([(0, 0.5), (0.5, 0.5), True, None] in self.test_wmap.pixels)
        self.assertTrue([(0.5, 0.5), (0.5, 0.5), False, None] in self.test_wmap.pixels)
        self.assertTrue([(0, 0), (0.5, 0.5), False, None] in self.test_wmap.pixels)
        self.assertTrue([(0.5, 0), (0.5, 0.5), None, "NULL"] in self.test_wmap.pixels)

    def test_gen_map_hex_bins(self):
        """Test the gen_map method."""
        self.test_wmap.device_attr = {}
        self.test_wmap.device_attr[CHIP_SIZE] = [1, 1]
        self.test_wmap.device_attr["rows"] = 2
        self.test_wmap.device_attr["cols"] = 2
        self.test_wmap.bin_codes = {
            "00": {"status": True, "desc": None, "count": "1"},
            "11": {"status": False, "desc": None, "count": "2"},
            "22": {"status": None, "desc": "NULL", "count": "3"},
        }
        self.test_wmap._map_data = {
            "Device": {
                "@BinType": "HexaDecimal",
                "Data": {
                    "Row": ["0011", "1122"],
                },
            }
        }

        self.test_wmap.gen_map()
        self.assertTrue([(0, 0.5), (0.5, 0.5), True, None] in self.test_wmap.pixels)
        self.assertTrue([(0.5, 0.5), (0.5, 0.5), False, None] in self.test_wmap.pixels)
        self.assertTrue([(0, 0), (0.5, 0.5), False, None] in self.test_wmap.pixels)
        self.assertTrue([(0.5, 0), (0.5, 0.5), None, "NULL"] in self.test_wmap.pixels)

    def test_gen_map_bad_units(self):
        """Test the gen_map method with incorrectly spec'd bins."""
        self.test_wmap.device_attr = {}
        self.test_wmap.device_attr[CHIP_SIZE] = [1, 1]
        self.test_wmap.device_attr["rows"] = 2
        self.test_wmap.device_attr["cols"] = 2
        self.test_wmap.bin_codes = {
            "000": {"status": True, "desc": None, "count": "1"},
            "111": {"status": False, "desc": None, "count": "2"},
            "222": {"status": None, "desc": "NULL", "count": "3"},
        }
        self.test_wmap._map_data = {
            "Device": {
                "@BinType": "Decimal",
                "Data": {
                    "Row": ["0011", "1122"],
                },
            }
        }

        with self.assertRaises(KeyError):
            self.test_wmap.gen_map()


class MockWaferMap(wafermap.WaferMap):
    """Mock of a wafermap class."""

    def __init__(self):
        """Override init of wafermap."""
        pass



"""Test wafermap with dummy data."""

import os
import unittest
from waferview import wafermap
from waferview.gui.constants import (
    WAFER_ID,
    LOT_ID,
    WAFER_SIZE,
    CHIP_SIZE,
    PRODUCT_ID,
    CREATE_DATE,
)


TEST_PATH = os.path.split(os.path.abspath(__file__))[0]


class TestWaferMapData(unittest.TestCase):
    """Test the WaferMap class on real XML data."""

    def test_semi_g85_1101_minimal(self):
        """Test G85-1101 minimal standard."""
        TEST_XML = os.path.join(TEST_PATH, "xml/SEMI_G85/SEMI_G85_1101_MIN.xml")
        test_wmap = wafermap.WaferMap(TEST_XML)
        self.assertTrue("00" in test_wmap.bin_codes)
        self.assertTrue("DE" in test_wmap.bin_codes)
        self.assertTrue("AD" in test_wmap.bin_codes)
        self.assertTrue("FF" in test_wmap.bin_codes)
        self.assertEqual(test_wmap.device_attr["rows"], 60)
        self.assertEqual(test_wmap.device_attr["cols"], 60)
        self.assertTrue(test_wmap.bin_codes["00"]["status"])
        self.assertFalse(test_wmap.bin_codes["DE"]["status"])
        self.assertFalse(test_wmap.bin_codes["AD"]["status"])
        self.assertEqual(test_wmap.bin_codes["FF"]["status"], None)
        self.assertEqual(test_wmap.bin_codes["00"]["count"], "2765")
        self.assertEqual(test_wmap.bin_codes["DE"]["count"], "38")
        self.assertEqual(test_wmap.bin_codes["AD"]["count"], "5")
        self.assertEqual(len(test_wmap.pixels), 3600)

    def test_semi_g85_1101_full(self):
        """Test G85-1101 full standard."""
        TEST_XML = os.path.join(TEST_PATH, "xml/SEMI_G85/SEMI_G85_1101_ALL.xml")
        test_wmap = wafermap.WaferMap(TEST_XML)
        self.assertTrue("00" in test_wmap.bin_codes)
        self.assertTrue("DE" in test_wmap.bin_codes)
        self.assertTrue("AD" in test_wmap.bin_codes)
        self.assertTrue("FF" in test_wmap.bin_codes)
        self.assertEqual(test_wmap.device_attr["rows"], 60)
        self.assertEqual(test_wmap.device_attr["cols"], 60)
        self.assertTrue(test_wmap.bin_codes["00"]["status"])
        self.assertFalse(test_wmap.bin_codes["DE"]["status"])
        self.assertFalse(test_wmap.bin_codes["AD"]["status"])
        self.assertEqual(test_wmap.bin_codes["FF"]["status"], None)
        self.assertEqual(test_wmap.bin_codes["00"]["count"], "2765")
        self.assertEqual(test_wmap.bin_codes["DE"]["count"], "38")
        self.assertEqual(test_wmap.bin_codes["AD"]["count"], "5")
        self.assertEqual(len(test_wmap.pixels), 3600)

        exp_attr = {
            WAFER_ID: "ABCD123",
            LOT_ID: "DEADBEEF",
            WAFER_SIZE: 300,
            CHIP_SIZE: [5000, 5000],
            PRODUCT_ID: "FOOBAR",
            CREATE_DATE: "20231028123456",
            "rows": 60,
            "cols": 60,
        }
        self.assertDictEqual(test_wmap.device_attr, exp_attr)



"""GUI module."""
import importlib.metadata
import os
import wx
import wx.adv
import wx.lib.scrolledpanel as scrolled
from waferview.gui import semimap
from waferview.gui import constants

__version__ = importlib.metadata.version("wafer-view")


def run():
    """Run the GUI."""
    app = wx.App()
    top = AppTop("Wafer-View: Open Source Wafer Map Viewer Utility")
    top.Show()
    app.MainLoop()


class AppTop(wx.Frame):
    """Main application."""

    def __init__(self, title):
        """Initialize the app."""
        wx.Frame.__init__(self, None, title=title)
        self.window()
        self.initialize()

    def initialize(self):
        """Create all panels and frames for the GUI."""
        self.create_panels()
        self.create_menu()
        self.create_status()
        self.create_controls()
        self.create_viewer()
        self.left_panel.SetSizer(self.sizers["left"])
        self.right_panel.SetSizer(self.sizers["right"])
        self.grid_panel.SetSizerAndFit(self.sizers["grid"])
        self.legend_panel.SetSizer(self.sizers["legend"])

    def window(self):
        """Set the window size."""
        (screen_width, screen_height) = wx.DisplaySize()
        # Limit to 4:3 aspect ratio
        self.xSize = int(
            screen_height * constants.WINDOW_SCALE * constants.ASPECT_RATIO
        )
        self.ySize = int(screen_height * constants.WINDOW_SCALE)
        self.SetSize(wx.Size(self.xSize, self.ySize))

    def get_panel_sizes(self):
        """Get panel sizes based on screen size."""
        self.panel_size = self.GetSize()
        # Limit to square aspect ratio
        self.panel_size[0] = int(self.panel_size[1])

        # Make viewer square
        viewerX = (
            self.panel_size[1] * constants.VIEWER_SCALE - 2 * constants.BORDER_SIZE
        )
        viewerY = (
            self.panel_size[1] * constants.VIEWER_SCALE - 2 * constants.BORDER_SIZE
        )
        dataX = self.panel_size[0] * constants.DATA_SCALE - 2 * constants.BORDER_SIZE
        dataY = self.panel_size[1] * constants.DATA_SCALE - 2 * constants.BORDER_SIZE

        self.grid_size = (dataX, int(self.panel_size[1] * constants.GRID_SCALE))
        self.legend_size = (dataX, int(self.panel_size[1] * constants.LEGEND_SCALE))
        self.control_size = (dataX, 50)

        self.legend_pos = (
            constants.BORDER_SIZE + 5,
            self.legend_size[1] + constants.BORDER_SIZE,
        )

        self.viewer_size = (viewerX, viewerY)
        self.data_size = (dataX, dataY)

    def create_panels(self):
        """Create panel structure and sizer elements."""
        # First the main panel with left and right sub-panels
        self.top_panel = wx.Panel(self)
        self.get_panel_sizes()

        self.left_panel = wx.Panel(
            self.top_panel, size=self.data_size, name="Wafer Data"
        )
        self.right_panel = wx.Panel(
            self.top_panel, size=self.panel_size, name="Wafer Map"
        )

        # In the left panel, we have one panel for the grid and one for the legend
        self.grid_panel = wx.Panel(
            self.left_panel,
            size=self.grid_size,
            name="Data Grid",
        )

        self.control_panel = wx.Panel(
            self.left_panel, size=self.control_size, name="Control Panel."
        )

        # Add main panels to the primary sizer
        self.main_sizer = wx.BoxSizer(wx.HORIZONTAL)
        self.main_sizer.Add(
            self.left_panel, 0, wx.EXPAND | wx.ALL, border=constants.BORDER_SIZE
        )
        self.main_sizer.Add(
            self.right_panel, 1, wx.EXPAND | wx.ALL, border=constants.BORDER_SIZE
        )
        self.top_panel.SetSizer(self.main_sizer)

        # Set background colors
        self.right_panel.SetBackgroundColour(constants.NULL_COLOR)
        self.right_panel.SetWindowStyle(wx.BORDER_THEME)

        self.left_panel.SetBackgroundColour("#AAAAAA")
        self.left_panel.SetWindowStyle(wx.BORDER_THEME)

        self.grid_panel.SetBackgroundColour("#AAAAAA")
        self.grid_panel.SetWindowStyle(wx.BORDER_THEME)

        # Create sizers for other panels
        self.sizers = {
            "left": wx.FlexGridSizer(rows=3, cols=1, vgap=1, hgap=1),
            "right": wx.BoxSizer(),
            "grid": wx.BoxSizer(),
        }

        # Draw the grid and legend panels at startup
        self.sizers["left"].Add(self.grid_panel, 1, wx.EXPAND | wx.ALL, border=1)

    def create_menu(self):
        """Create the menu bar."""
        self.menubar = MenuBar(self)
        self.SetMenuBar(self.menubar)

    def create_viewer(self):
        """Create the wafer map viewer."""
        try:
            pixels = self.viewer.pixel_elements
            colors = self.viewer.color_map
            scale = self.viewer.scale
            self.viewer.Destroy()
        except AttributeError:
            pixels = {}
            colors = {}
        self.create_legend()
        self.viewer = semimap.Viewer(
            self, self.right_panel, self.viewer_size[0], self.viewer_size[1]
        )
        try:
            self.viewer.scale = scale
        except UnboundLocalError:
            self.viewer.pixel_elements = pixels
        self.viewer.color_map = colors
        self.sizers["right"].Add(self.viewer, 1, wx.EXPAND | wx.ALL)

    def create_controls(self):
        """Create the wafermap controls section."""
        self.sizers["control"] = wx.FlexGridSizer(rows=2, cols=4, gap=wx.Size(1, 3))
        btn_down = wx.Button(self.control_panel, 1, "Down", size=(10, 10))
        btn_up = wx.Button(self.control_panel, 1, "Up")
        btn_left = wx.Button(self.control_panel, 1, "Left")
        btn_right = wx.Button(self.control_panel, 1, "Right")
        btn_zoom_out = wx.Button(self.control_panel, 1, "Zoom -")
        btn_zoom_in = wx.Button(self.control_panel, 1, "Zoom +")
        btn_fit = wx.Button(self.control_panel, 1, "Fit")

        self.sizers["control"].Add(btn_down, -1, wx.EXPAND | wx.ALL, border=1)
        self.sizers["control"].Add(btn_up, -1, wx.EXPAND | wx.ALL, border=1)
        self.sizers["control"].Add(btn_left, -1, wx.EXPAND | wx.ALL, border=1)
        self.sizers["control"].Add(btn_right, -1, wx.EXPAND | wx.ALL, border=1)
        self.sizers["control"].Add(btn_zoom_out, -1, wx.EXPAND | wx.ALL, border=1)
        self.sizers["control"].Add(btn_zoom_in, -1, wx.EXPAND | wx.ALL, border=1)
        self.sizers["control"].Add(btn_fit, -1, wx.EXPAND | wx.ALL, border=1)

        self.control_panel.SetSizer(self.sizers["control"])
        self.sizers["left"].Add(self.control_panel, 1, wx.EXPAND | wx.ALL, border=1)

        btn_down.Bind(wx.EVT_BUTTON, lambda event: self.set_offset(event, (0, 10)))
        btn_up.Bind(wx.EVT_BUTTON, lambda event: self.set_offset(event, (0, -10)))
        btn_left.Bind(wx.EVT_BUTTON, lambda event: self.set_offset(event, (-10, 0)))
        btn_right.Bind(wx.EVT_BUTTON, lambda event: self.set_offset(event, (10, 0)))
        btn_zoom_out.Bind(wx.EVT_BUTTON, lambda event: self.set_scale(event, 0.9))
        btn_zoom_in.Bind(wx.EVT_BUTTON, lambda event: self.set_scale(event, 1.1))
        btn_fit.Bind(wx.EVT_BUTTON, lambda event: self.set_scale(event, 0))

    def create_legend(self):
        """Create the legend section."""
        try:
            self.legend_panel.Destroy()
        except AttributeError:
            pass
        self.legend_panel = scrolled.ScrolledPanel(
            self.left_panel,
            size=self.legend_size,
            pos=self.legend_pos,
            name="Legend",
        )
        self.sizers["left"].Add(self.legend_panel, 1, wx.EXPAND | wx.ALL, border=2)
        self.sizers["legend"] = wx.BoxSizer(wx.VERTICAL)
        font = wx.Font(wx.FontInfo(16).Bold())
        legend_text = wx.StaticText(
            self.legend_panel, label="Wafer Map Legend", style=wx.ALIGN_CENTER
        )
        legend_text.SetFont(font)
        self.sizers["legend"].Add(legend_text, 1, wx.ALL | wx.ALIGN_CENTER, border=5)

    def create_status(self):
        """Create the data grid table."""
        self.data_grid = semimap.DataGrid(self.grid_panel)
        self.sizers["grid"].Add(self.data_grid, 1, wx.ALL)

    def map_filename(self):
        """Return the wafermap file name."""
        return self.filemenu.file_name

    def set_scale(self, event, zoom_type):
        """Set wafermap scaling based on zoom button input."""
        if zoom_type == 0:
            self.viewer.zoom_factor = 1
            self.viewer.xorigin = 0
            self.viewer.yorigin = 0
            return
        self.viewer.zoom_factor = zoom_type * self.viewer.zoom_factor

    def set_offset(self, event, offset):
        """Set the frame offset."""
        self.viewer.xorigin = self.viewer.xorigin + offset[0] * self.viewer.zoom_factor
        self.viewer.yorigin = self.viewer.yorigin + offset[1] * self.viewer.zoom_factor
        self.viewer.OnPaint(None)


class MenuBar(wx.MenuBar):
    """File Menu Bar."""

    def __init__(self, parent):
        """Initialize the menu bar."""
        wx.MenuBar.__init__(self)
        self.parent = parent
        self.initialize()

    def initialize(self):
        """Generate the menu bar structure."""
        filemenu = wx.Menu()
        self.Append(filemenu, "&File")
        filemenu.Append(wx.ID_OPEN, "O&pen\tCtrl-O")
        filemenu.Append(wx.ID_EXIT, "E&xit\tAlt-X", "Close window and exit program")
        filemenu.Append(wx.ID_SAVE, "S&ave\tCtrl-S", "Save wafermap image")
        self.parent.Bind(wx.EVT_MENU, self.save_image, id=wx.ID_SAVE)
        self.parent.Bind(wx.EVT_MENU, self.file_browser, id=wx.ID_OPEN)

        helpmenu = wx.Menu()
        self.Append(helpmenu, "&Help")
        helpmenu.Append(wx.ID_ABOUT, "", "About Waferview")
        self.parent.Bind(wx.EVT_MENU, self.about_screen, id=wx.ID_ABOUT)

    def about_screen(self, event):
        """Open the About dialog on event."""
        about = wx.adv.AboutDialogInfo()
        about.SetName("Wafer-View")
        about.SetVersion(__version__)
        about.SetLicense("Apache 2.0 <https://github.com/fronzbot/wafer-view/LICENSE>")
        about.SetDescription("Open source wafer map viewer utility")
        about.SetCopyright("(C) 2023")
        about.SetWebSite("http://github.com/fronzbot/wafer-view")
        about.AddDeveloper("Kevin Fronczak <kfronczak@gmail.com>")

        wx.adv.AboutBox(about)

    def save_image(self, event):
        """Save image drawn on wafermap viewer."""
        with wx.DirDialog(
            self,
            message="Choose save directory",
            defaultPath=".",
        ) as dir_dialog:
            if dir_dialog.ShowModal() == wx.ID_CANCEL:
                return
            dir_name = dir_dialog.GetPath()

        dc = self.parent.viewer.OnPaint(None)
        size = self.parent.viewer.GetSize()
        bmp = wx.Bitmap(size.width, size.height)
        memdc = wx.MemoryDC()
        memdc.SelectObject(bmp)

        memdc.Blit(0, 0, size.width, size.height, dc, 0, 0)
        memdc.SelectObject(wx.NullBitmap)

        img = bmp.ConvertToImage()
        file_name = os.path.join(dir_name, "wafermap.bmp")
        img.SaveFile(file_name, wx.BITMAP_TYPE_PNG)

        notif = wx.MessageDialog(
            None,
            f"{file_name}",
            caption="Wafer Map Image Saved!",
            style=wx.OK | wx.ALIGN_RIGHT,
        )
        notif.ShowModal()

    def file_browser(self, event):
        """Open the file browser on event."""
        with wx.FileDialog(
            self,
            message="Choose file",
            defaultDir=".",
            defaultFile="",
            wildcard="XML files (*.xml)|*.xml",
            style=wx.FD_OPEN | wx.FD_FILE_MUST_EXIST,
        ) as file_dialog:
            if file_dialog.ShowModal() == wx.ID_CANCEL:
                return
            self.file_name = file_dialog.GetPath()

        # Once a file is selected, we need to generate the map and viewer
        self.parent.create_viewer()
        self.parent.viewer.generate_map(self.file_name)



"""Creates memory structure for wafer map."""

import re
import xml.etree.ElementTree as ET
import xmltodict
from waferview.gui.constants import (
    SUPPORTED_FORMATS,
    WAFER_ID,
    LOT_ID,
    WAFER_SIZE,
    CHIP_SIZE,
    PRODUCT_ID,
    CREATE_DATE,
    DEFAULT_WAFER_SIZE,
    PASS,
    NULL,
)


class WaferMap:
    """Representation of a wafer map."""

    def __init__(self, xmlfile):
        """Initialize a wafer map structure."""
        self.parse(xmlfile)
        self.check_format()
        self.get_attributes()
        self.get_codes()
        self.gen_map()

    def parse(self, xmlfile):
        """Parse an xml wafer map."""
        tree = ET.iterparse(xmlfile)
        # Strip namespace from XML file
        for _, elem in tree:
            _, _, elem.tag = elem.tag.rpartition("}")
        xml_data = tree.root
        xmlstr = ET.tostring(xml_data, encoding="utf-8", method="xml")
        xml_dict = xmltodict.parse(xmlstr)
        self._map_data = xml_dict["Map"]

    def check_format(self):
        """Verify format is supported by library."""
        self.is_valid = False
        self.format = self._map_data.get("@FormatRevision", None)
        if self.format in SUPPORTED_FORMATS:
            self.is_valid = True

    def get_attributes(self):
        """Retrieve all wafer attributes."""
        device_data = self._map_data["Device"]
        self.device_attr = {
            WAFER_ID: self._map_data.get("@WaferId", None),
            LOT_ID: device_data.get("@LotId", None),
            WAFER_SIZE: float(device_data.get("@WaferSize", DEFAULT_WAFER_SIZE)),
            CHIP_SIZE: [
                float(device_data.get("@DeviceSizeX", 0)),
                float(device_data.get("@DeviceSizeY", 0)),
            ],
            PRODUCT_ID: device_data.get("@ProductId", None),
            CREATE_DATE: device_data.get("@CreateDate", None),
            "rows": int(device_data.get("@Rows", 1)),
            "cols": int(device_data.get("@Columns", 1)),
        }

    def get_codes(self):
        """Get all bin codes."""
        bins = self._map_data["Device"]["Bin"]
        null_bin = self._map_data["Device"]["@NullBin"]
        self.bin_codes = {}
        bin_incr = 0
        for code in bins:
            code_val = code["@BinCode"]
            code_pass = code["@BinQuality"] == PASS
            code_desc = code.get("@BinDescription", str(bin_incr))
            code_count = code["@BinCount"]
            if code["@BinQuality"] == NULL:
                code_pass = None
            self.bin_codes[code_val] = {
                "status": code_pass,
                "desc": code_desc,
                "count": code_count,
            }
            bin_incr += 1

        if null_bin not in self.bin_codes:
            self.bin_codes[null_bin] = {
                "status": None,
                "desc": NULL,
                "count": None,
            }

    def gen_map(self):
        """Generate a wafer map with data and coordinates."""
        if self.device_attr[CHIP_SIZE] == [0, 0]:
            # Need to use rows and columns to guess size
            self.device_attr[CHIP_SIZE][0] = (
                1000 * self.device_attr[WAFER_SIZE] / self.device_attr["cols"]
            )
            self.device_attr[CHIP_SIZE][1] = (
                1000 * self.device_attr[WAFER_SIZE] / self.device_attr["rows"]
            )

        # Normalize locations to a 0 to 1 grid with 0,0 at bottom left and
        # 1,1, at top right
        xmax = self.device_attr[CHIP_SIZE][0] * self.device_attr["cols"]
        ymax = self.device_attr[CHIP_SIZE][1] * self.device_attr["rows"]
        xstep = self.device_attr[CHIP_SIZE][0] / xmax
        ystep = -1 * self.device_attr[CHIP_SIZE][1] / ymax

        self.pixels = []

        xloc = 0
        yloc = 1

        bin_type = self._map_data["Device"]["@BinType"]
        find_str = ".."
        if bin_type == "Decimal":
            find_str = "..."

        for row in self._map_data["Device"]["Data"]["Row"]:
            # Split data into one byte chunks
            row_bins = re.findall(find_str, row)
            for data in row_bins:
                status = self.bin_codes[data]["status"]
                desc = self.bin_codes[data]["desc"]
                coord = (xloc, yloc + ystep)
                size = (xstep, -1 * ystep)
                self.pixels.append([coord, size, status, desc])
                xloc += xstep
            xloc = 0
            yloc += ystep
